from rest_framework import viewsets
from rest_framework.exceptions import APIException
from rest_framework.decorators import action
from rest_framework.authtoken.views import ObtainAuthToken
from rest_framework.authtoken.models import Token


try:
    from ..response import ApiResponse,ApiState
except:
    from ...rest.response import ApiResponse, ApiState



class ApiAuthView(viewsets.ViewSetMixin,ObtainAuthToken):

    Token = None
    @property
    def model(self):
        return self.Token.model


    def loginResponse(self,user,token):
        data = {'id':user.id}
        data['token'] = token
        return data

    def registResponse(self,user,token):
        return {'id': user.id, 'token': token}

    @action(methods=['post','options','get'], detail=False)
    def register(self, request):
        print("======DD",self.Token)
        if self.model is None:
            return ApiResponse(ApiState.exception, "login error: ApiAuthView can not be None (authModel inherit from User model class )", "")
        model = self.model
        try:
            #该处代码参照ObtainAuthToken
            serializer = self.serializer_class(data=request.data,
                                               context={'request': request})
            serializer.is_valid(raise_exception=False)
            data = serializer.data

            username = data.get('username','')
            password = data.get('password','')
            if username == '':
                return ApiResponse(ApiState.failed, "请输入用户名", {})
            if password == '':
                return ApiResponse(ApiState.failed, "请输入有效的密码", {})
            print('-----001')

            us = model.objects.filter(username=data['username'])
            if us.count() > 0:
                print('-----002')

                return ApiResponse(ApiState.failed, '用户名已被注册', {})
            print('-----00-02021',model.objects.all())

            newObj = model.objects.create(username=username)
            print('-----model.objects-02021',model.objects)

            newObj.set_password(password)
            newObj.save()
            print('-----003')

            token, created = self.Token.objects.get_or_create(user=newObj)
            print('-----004')

            data = self.registResponse(newObj, token.key)
            return ApiResponse(ApiState.success, "注册成功", data)
        except Exception as e:
            raise APIException(e)


    @action(methods=['post','options','get'], detail=False)
    def login(self, request, *args, **kwargs):
        # 该处代码参照ObtainAuthToken
        # import time
        # time.sleep(1)

        serializer = self.serializer_class(data=request.data,
                                           context={'request': request})
        try:
            serializer.is_valid(raise_exception=True)
        except Exception as e:
            return ApiResponse(ApiState.failed, "用户名或密码错误", {})

        if self.authModel is None:
            return ApiResponse(ApiState.exception, "login error: TokenAuthView can not be None (authModel inherit from User model class )", "")
        try:
            baseUser = serializer.validated_data['user']
            token, created = Token.objects.get_or_create(user=baseUser)
            data = self.loginResponse(baseUser,token.key)

            return ApiResponse(ApiState.success, "登录成功", data)
        except Exception as e:
            return ApiResponse(ApiState.failed, "用户名或密码错误", {})





class UserAuthView(viewsets.ViewSetMixin,ObtainAuthToken):
    authModel = None


    def loginResponse(self,user,token):
        data = {'id':user.id}
        data['token'] = token
        return data

    def registResponse(self,user,token):
        return {'id': user.id, 'token': token}

    @action(methods=['post','options','get'], detail=False)
    def register(self, request):

        if self.authModel is None:
            return ApiResponse(ApiState.exception, "login error: TokenAuthView can not be None (authModel inherit from User model class )", "")
        model = self.authModel
        try:
            #该处代码参照ObtainAuthToken
            serializer = self.serializer_class(data=request.data,
                                               context={'request': request})
            serializer.is_valid(raise_exception=False)
            data = serializer.data

            username = data.get('username','')
            password = data.get('password','')
            if username == '':
                return ApiResponse(ApiState.failed, "请输入用户名", {})
            if password == '':
                return ApiResponse(ApiState.failed, "请输入有效的密码", {})


            us = model.objects.filter(username=data['username'])
            if us.count() > 0:
                return ApiResponse(ApiState.failed, '用户名已被注册', {})

            newObj = model.objects.create(username=username)
            newObj.set_password(password)
            newObj.save()
            token, created = Token.objects.get_or_create(user=newObj)

            data = self.registResponse(newObj, token.key)

            return ApiResponse(ApiState.success, "注册成功", data)
        except Exception as e:
            raise APIException(e)


    @action(methods=['post','options','get'], detail=False)
    def login(self, request, *args, **kwargs):
        # 该处代码参照ObtainAuthToken
        # import time
        # time.sleep(1)

        serializer = self.serializer_class(data=request.data,
                                           context={'request': request})
        try:
            serializer.is_valid(raise_exception=True)
        except Exception as e:
            return ApiResponse(ApiState.failed, "用户名或密码错误", {})

        if self.authModel is None:
            return ApiResponse(ApiState.exception, "login error: TokenAuthView can not be None (authModel inherit from User model class )", "")
        try:
            baseUser = serializer.validated_data['user']
            token, created = Token.objects.get_or_create(user=baseUser)
            data = self.loginResponse(baseUser,token.key)

            return ApiResponse(ApiState.success, "登录成功", data)
        except Exception as e:
            return ApiResponse(ApiState.failed, "用户名或密码错误", {})





# Generated by Django 2.1.7 on 2019-02-12 09:19

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='CodeRecord',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('VALID', models.BooleanField(default=True)),
                ('mobile', models.CharField(max_length=11, verbose_name='手机号')),
                ('code', models.CharField(max_length=6, verbose_name='验证码')),
                ('tag', models.CharField(default='unkown', max_length=32, verbose_name='标签')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



# Generated by Django 2.1.7 on 2019-02-12 09:19

from django.db import migrations, models
import django.db.models.deletion
import meapp.core.fields.ImageSetField
import meapp.core.model.Images


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='ImageModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('filename', models.CharField(max_length=100, verbose_name='文件名')),
                ('image', models.ImageField(storage=meapp.core.model.Images.ImageStorage(), upload_to='img/%Y/%m/%d')),
            ],
            options={
                'ordering': ['id'],
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Log',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('content', models.CharField(blank=True, max_length=255, verbose_name='日志内容')),
                ('time', models.DateTimeField(auto_now_add=True, verbose_name='时间')),
                ('mark', models.CharField(blank=True, max_length=255, verbose_name='操作者')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='OptionsModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=64, verbose_name='名称')),
                ('value', models.CharField(max_length=20, verbose_name='值')),
            ],
        ),
        migrations.CreateModel(
            name='TestModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('email', models.EmailField(blank=True, max_length=70, verbose_name='邮箱')),
                ('name', models.CharField(max_length=20, verbose_name='名称')),
                ('covers', meapp.core.fields.ImageSetField.ImageSetField(blank=True, verbose_name='封面')),
                ('sex', models.CharField(choices=[('boy', '男'), ('girl', '女'), ('unknown', '未知')], default='unknown', max_length=10, verbose_name='性别')),
                ('hobbies', models.CharField(blank=True, max_length=50, verbose_name='爱好')),
                ('time', models.TimeField(verbose_name='时间')),
                ('birthday', models.DateField(verbose_name='生日')),
                ('over_time', models.DateTimeField(blank=True, verbose_name='过期时间')),
            ],
        ),
        migrations.CreateModel(
            name='WXAuthLog',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('openid', models.CharField(max_length=256, verbose_name='微信openid')),
                ('access_token', models.CharField(max_length=256, verbose_name='access_token')),
                ('refresh_token', models.CharField(max_length=256, verbose_name='refresh_token')),
                ('create_time', models.DateTimeField(auto_now_add=True, verbose_name='创建时间')),
                ('VALID', models.BooleanField(default=True)),
            ],
        ),
        migrations.CreateModel(
            name='WXUserAuth',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('openid', models.CharField(max_length=256, verbose_name='微信openid')),
                ('code', models.CharField(blank=True, max_length=256, verbose_name='code')),
                ('access_token', models.CharField(blank=True, max_length=256, verbose_name='access_token')),
                ('refresh_token', models.CharField(blank=True, max_length=256, verbose_name='refresh_token')),
                ('create_time', models.DateTimeField(auto_now_add=True, verbose_name='创建时间')),
                ('update_time', models.DateTimeField(auto_now=True, verbose_name='更新时间')),
                ('VALID', models.BooleanField(default=True)),
            ],
        ),
        migrations.AddField(
            model_name='wxauthlog',
            name='user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='core.WXUserAuth', verbose_name='用户'),
        ),
    ]



# Generated by Django 2.0 on 2019-02-12 06:20

import eeapp.base.models
import django.contrib.auth.validators
from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Address',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('order', models.IntegerField(default=0, verbose_name='排序')),
                ('street', models.CharField(default='', max_length=300)),
                ('address', models.CharField(default='', max_length=300)),
                ('post_code', models.CharField(default='', max_length=10)),
            ],
        ),
        migrations.CreateModel(
            name='Admin',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('password', models.CharField(max_length=128, verbose_name='password')),
                ('last_login', models.DateTimeField(blank=True, null=True, verbose_name='last login')),
                ('display_name', models.CharField(default='', max_length=255, verbose_name='昵称')),
                ('VALID', models.BooleanField(default=True)),
                ('role', models.CharField(default='base', max_length=10)),
                ('wxopenid', models.CharField(default='', max_length=64)),
                ('username', models.CharField(error_messages={'unique': 'A user with that username already exists.'}, help_text='Required. 150 characters or fewer. Letters, digits and @/./+/-/_ only.', max_length=150, unique=True, validators=[django.contrib.auth.validators.UnicodeUsernameValidator()], verbose_name='username')),
                ('first_name', models.CharField(blank=True, max_length=30, verbose_name='first name')),
                ('last_name', models.CharField(blank=True, max_length=150, verbose_name='last name')),
                ('email', models.EmailField(blank=True, max_length=254, verbose_name='email address')),
                ('is_staff', models.BooleanField(default=False, help_text='Designates whether the user can log into this admin site.', verbose_name='staff status')),
                ('is_active', models.BooleanField(default=True, help_text='Designates whether this user should be treated as active. Unselect this instead of deleting accounts.', verbose_name='active')),
                ('date_joined', models.DateTimeField(default=django.utils.timezone.now, verbose_name='date joined')),
                ('mobile', models.CharField(max_length=20, verbose_name='手机号')),
            ],
            options={
                'verbose_name': 'user',
                'verbose_name_plural': 'users',
                'abstract': False,
            },
            managers=[
                ('objects', eeapp.base.models.UManager()),
            ],
        ),
        migrations.CreateModel(
            name='AdminToken',
            fields=[
                ('key', models.CharField(max_length=40, primary_key=True, serialize=False, verbose_name='Key')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('user', models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='auth_token', to='eeapp.jsonapi.location.Admin', verbose_name='User')),
            ],
            options={
                'verbose_name': 'Token',
                'verbose_name_plural': 'Tokens',
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Area',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('area_id', models.CharField(default=None, max_length=50, verbose_name='地区编号')),
                ('area', models.CharField(default=None, max_length=60, verbose_name='区')),
                ('order', models.IntegerField(default=0, verbose_name='排序')),
            ],
        ),
        migrations.CreateModel(
            name='City',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('city_id', models.CharField(default=None, max_length=6, verbose_name='城市编号')),
                ('city', models.CharField(default=None, max_length=50, verbose_name='市/县')),
                ('order', models.IntegerField(default=0, verbose_name='排序')),
            ],
        ),
        migrations.CreateModel(
            name='Province',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('province_id', models.CharField(default=None, max_length=6, verbose_name='省份编号')),
                ('province', models.CharField(default=None, max_length=40, verbose_name='省份')),
                ('order', models.IntegerField(default=0, verbose_name='排序')),
            ],
        ),
        migrations.AddField(
            model_name='city',
            name='father',
            field=models.ForeignKey(default=None, on_delete=django.db.models.deletion.DO_NOTHING, related_name='cities', to='eeapp.jsonapi.location.Province', verbose_name='所属省份'),
        ),
        migrations.AddField(
            model_name='area',
            name='father',
            field=models.ForeignKey(default=None, on_delete=django.db.models.deletion.DO_NOTHING, related_name='areas', to='eeapp.jsonapi.location.City', verbose_name='所属城市'),
        ),
        migrations.AddField(
            model_name='address',
            name='area',
            field=models.ForeignKey(on_delete=django.db.models.deletion.DO_NOTHING, to='eeapp.jsonapi.location.Area'),
        ),
    ]
